/*
 * HTTSQUASH.H
 * Contains ip range structure and the networking
 * fuctions used by the program.
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef HTTSQUASH_LIB
#define HTTSQUASH_LIB

#include <stdio.h>
#include <string.h>
#ifdef WIN32
#include <windows.h>
#include <winsock2.h>
#else
#include <netdb.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#endif

#include "argparser.h"

#ifdef WIN32
WORD version;
WSADATA WSAData;
#endif

#define MAX_PROC 6

int count=0;

/*
 * Portable open socket
 * IPv4 and IPv6 compatible
 */
int opensock(char *host, short int port)
{    
    int sock;
    char service[20];
    
    struct addrinfo hints, *res;
    
    struct sockaddr_in *sin;
    struct sockaddr_in6 *sin6;

    #ifdef WIN32
        version=MAKEWORD(1,1);
        WSAStartup(version, &WSAData);
    #endif


    /* RESOLVE DESTINATION */
    memset(&hints, 0, sizeof(hints));
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_family = PF_UNSPEC;
    
    sprintf(service, "%d", port);
    
    if(getaddrinfo(host, service, &hints, &res)!=0)
        return -1;
    
    
    /* OPEN SOCKET */

    if ((sock=socket(res->ai_family, res->ai_socktype, res->ai_protocol)) == -1)
        return -1;

    
    /* CONNECT TO HOST */
    
    if (connect(sock, res->ai_addr, res->ai_addrlen) == -1) {
        freeaddrinfo(res);
        return -3;
    }    
        
    freeaddrinfo(res);

    return sock;
}

/*
 * Portable close socket
 */
void closesock(int sock)
{
    #ifdef WIN32
        closesocket(sock);
    #else
        close(sock);
    #endif
}

/*
 * Portable send data
 */
void senddata(int sock, const char *data)
{
    send(sock,data,strlen(data),0);
}

/*
 * Portable recv data
 */
char *recvdata(int sock, int max_data)
{
    int res=0;
    char *buffer=NULL;
    buffer = (char *) malloc(max_data);

    res = recv(sock,buffer,max_data,0);
    buffer[res] = '\0';

    return buffer;
}

/*
 * Portable recv data loop
 */
char *recvdataloop(int sock, int max_data)
{
    int res=0, val=0;
    char *pb=NULL, *buffer=NULL;
    pb= (char *) malloc(max_data+1);

    while(res = recv(sock,pb,max_data,0)) {
        buffer = (char *) realloc(buffer, val+res);
        buffer[val] = '\0';
        val+=res;
        
        pb[res] = '\0';
        strcat(buffer,pb);
        pb[0] = '\0';
    }

    free(pb);

    return buffer;
}

#endif /* HTTSQUASH_LIB */

