/*
 * ReverseRaider, domain scanner.
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 * 
 * From an idea of Jeremy Brown --- http://jbrownsec.blogspot.com/
 *
 * Thanks to:
 * Martin Holst Swende <martin.holst_swende@msc.se>
 * for wordlistscanner() function
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "reverseraider.h"

#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <ctype.h>
#include <arpa/inet.h>
#include <sys/wait.h>

#define MAX_PROC 6

/*
 * Catch SIGALRM
 */
void catch_alarm (int sig)
{
    exit(0);
}

/*
 * Brute force wordlist scanner
 * 
 * Thanks to:
 * Martin Holst Swende <martin.holst_swende@msc.se>
 */
void wordlistscanner(struct options *opt)
{
    char host[MAX_ARG];
    char prefix[MAX_ARG];
    
    char ipstr[MAX_ARG];
    char previpstr[MAX_ARG];
    
    struct addrinfo hints, *res,*p;
    
    FILE *wordlist;
    
    int i=0, childs=0;
    
    int permcount = 40; // permutation counter
    
    void *addr=NULL;
    
    
    wordlist = fopen(opt->wordlist, "r");
    if(!wordlist) {
        fprintf(stderr, "Error opening %s\n", opt->wordlist);
        exit(0);
    }
  
    fflush(stdout);
    while(!feof(wordlist)) {
    
        // get prefix
        fgets(prefix, MAX_ARG, wordlist);
        
        // delete newlines
        for(i=0; prefix[i]!=0; i++)
            if (prefix[i]=='\n') {
            prefix[i]='\0';
            break;
        }
    
        memset(host, MAX_ARG, 0);
        
        //child (scan..)
        if(fork()==0) {
            
            memset(&hints, 0, sizeof(hints));
            snprintf(host, MAX_ARG, "%s.%s", prefix, opt->domain);
            
            // if address is found
            if(getaddrinfo(host, NULL, &hints, &res) == 0) {
            
                memset(&previpstr, 0, MAX_ARG);
                
                for(p = res; p != NULL; p = p->ai_next) {

                    // get the pointer to the address itself,
                    // different fields in IPv4 and IPv6
                    
                    if (p->ai_family == AF_INET) { // IPv4
                        struct sockaddr_in *ipv4 = (struct sockaddr_in *) p->ai_addr;
                        addr = &(ipv4->sin_addr);
                    }
                    
                    else if (p->ai_family == AF_INET6) { // IPv6
                        struct sockaddr_in6 *ipv6 = (struct sockaddr_in6 *)p->ai_addr;
                        addr = &(ipv6->sin6_addr);
                    }
                    
                    else {
                        continue;
                    }

                    // convert the IP to a string and print it:
                    inet_ntop(p->ai_family, addr, ipstr, sizeof ipstr);
                    
                    // compare with prev. IP
                    if(strcmp(previpstr, ipstr)) {
                        strncpy(previpstr, ipstr, MAX_ARG);
                    
                        printf("%s %s\n", host, ipstr);
                    }
                }

                freeaddrinfo(res); 
            }
            
            
            // Permutation?
            while(opt->permutation && permcount) {
                
                // Permutation: prefix-number-domain
                for(i=0; i<10; i++) {
                
                    memset(&hints,0,sizeof(hints));
                    
                    // permutation type
                    if (permcount > 30)
                        snprintf(host, MAX_ARG, "%s%d.%s", prefix, i, opt->domain);
                    else if (permcount > 20)
                        snprintf(host, MAX_ARG, "%s0%d.%s", prefix, i, opt->domain);
                    else if (permcount > 10)
                        snprintf(host, MAX_ARG, "%d%s.%s", i, prefix, opt->domain);
                    else if (permcount > 0)
                        snprintf(host, MAX_ARG, "0%d%s.%s", i, prefix, opt->domain);
            
                    // if address is found
                    if(getaddrinfo(host, NULL, &hints, &res) == 0) {
                    
                        memset(&previpstr, 0, MAX_ARG);
                    
                        for(p = res; p != NULL; p = p->ai_next) {

                            // get the pointer to the address itself,
                            // different fields in IPv4 and IPv6
                            
                            if (p->ai_family == AF_INET) { // IPv4
                                struct sockaddr_in *ipv4 = (struct sockaddr_in *) p->ai_addr;
                                addr = &(ipv4->sin_addr);
                            }
                            
                            else if (p->ai_family == AF_INET6) { // IPv6
                                struct sockaddr_in6 *ipv6 = (struct sockaddr_in6 *)p->ai_addr;
                                addr = &(ipv6->sin6_addr);
                            }
                            
                            else {
                                continue;
                            }

                            // convert the IP to a string and print it:
                            inet_ntop(p->ai_family, addr, ipstr, sizeof ipstr);
                            
                            // compare with prev. IP
                            if(strcmp(previpstr, ipstr)) {
                                strncpy(previpstr, ipstr, MAX_ARG);
                            
                                printf("%s %s\n", host, ipstr);
                            }

                        }

                        freeaddrinfo(res); 
                    }

                    fflush(stdout);
                    
                    permcount--;
                }
                            
            } // permutation end
            
            
            fflush(stdout);
            
            exit(0);
        }
        
        //daddy...
        else {
            if(childs>=MAX_PROC) for(childs; childs>0; childs--) wait(0);
        }

        childs++;
    }
    
    for(childs; childs>0; childs--) wait(0);

    fclose(wordlist);
}

/*
 * Reverse resolution scanner
 */
void reversescanner(struct options *opt)
{
    char addr[MAX_ARG];
    char result[MAX_ARG];

    struct sockaddr_in sa;
    struct sockaddr_in6 sa6;

    int start, end, curr, childs=0;

    start = opt->start;
    end = opt->end;
    curr = opt->start;

    while(curr <= end) {

        //child (scan..)
        if(fork()==0) {
        
            alarm(opt->time); //timer
            
            // current address
            if(opt->ipver==VER_IPV4)
                sprintf(addr, "%s.%u", opt->block, curr);
            else
                sprintf(addr, "%s:%x", opt->block, curr);
            
            // fill sockaddr struct
            if(opt->ipver==VER_IPV4) {
                memset(&sa, 0, sizeof(sa));
                
                sa.sin_family = AF_INET;
                if(inet_pton(AF_INET, addr, &(sa.sin_addr)) < 0) {
                    fprintf(stderr, "Address %s is not valid\n", addr);
                    exit(0);
                }
            }
            else {
                memset(&sa6, 0, sizeof(sa6));
                
                sa6.sin6_family = AF_INET6;
                
                if(inet_pton(AF_INET6, addr, &(sa6.sin6_addr)) < 0) {
                    fprintf(stderr, "Address %s is not valid\n", addr);
                    exit(0);
                }
            }
            
            // if address is found
            if(opt->ipver==VER_IPV4) {
                if(getnameinfo((void *) &sa, sizeof(sa), result, MAX_ARG, NULL, 0, NI_NOFQDN|NI_NAMEREQD) == 0) {

                    // Print result
                    printf("%s %s\n", addr, result);            
                }
            }
            else {
                if(getnameinfo((void *) &sa6, sizeof(sa6), result, MAX_ARG, NULL, 0, NI_NOFQDN|NI_NAMEREQD) == 0) {

                    // Print result
                    printf("%s %s\n", addr, result);            
                }
            }

            exit(0);
        }
        
        //daddy...
        else {
            if(childs>=MAX_PROC) for(childs; childs>0; childs--) wait(0);
        }

        childs++;
        curr++;
    }

    for(childs; childs>0; childs--) wait(0);
}

/*
 * Program init
 */
int main(int argc, char **argv)
{

    struct options opt;

    // Establish a handler for SIGALRM signals
    signal (SIGALRM, catch_alarm);
    signal (SIGTERM, catch_alarm);
    signal (SIGKILL, catch_alarm);
    
    // parse options
    parseopt(&opt, argc, argv);

    // Wordlist scanning
    if(opt.domain[0]!='\0') {
        wordlistscanner(&opt);
    }
       
    // Reverse scanning
    else {
        reversescanner(&opt);
    }
    
    return 0;
}
